@@ -2,7 +2,15 @@ from __future__ import annotations
 
 from .applications import Index
 from .exceptions import HTTPException
-from .field_functions import Body, Cookie, Header, Path, Query, Request
+from .parameters.field_functions import (
+    Body,
+    Cookie,
+    Depends,
+    Header,
+    Path,
+    Query,
+    Request,
+)
 from .requests import request, websocket
 from .responses import (
     FileResponse,
@@ -14,6 +22,7 @@ from .responses import (
     SendEventResponse,
     ServerSentEvent,
     StreamResponse,
+    TemplateResponse,
 )
 from .routing import HttpRoute, Routes, SocketRoute
 from .views import HttpView, SocketView, required_method
@@ -29,6 +38,7 @@ __all__ = [
     "Path",
     "Query",
     "Request",
+    "Depends",
     "HttpResponse",
     "FileResponse",
     "HTMLResponse",
