@@ -4,7 +4,6 @@ import copy
 import dataclasses
 import functools
 import inspect
-import sys
 import traceback
 from functools import reduce
 from pathlib import PurePath
@@ -13,9 +12,9 @@ from typing import (
     Any,
     Awaitable,
     Callable,
-    Dict,
     Iterable,
     List,
+    Mapping,
     NoReturn,
     Optional,
     Type,
@@ -23,14 +22,9 @@ from typing import (
     Union,
 )
 
-if sys.version_info[:2] < (3, 8):
-    from typing_extensions import Literal
-else:
-    from typing import Literal
-
-from baize.asgi import Receive, Scope, Send
 from baize.datastructures import URL
-from baize.typing import ASGIApp
+from baize.typing import ASGIApp, Receive, Scope, Send
+from typing_extensions import Literal
 
 from .debug import DebugMiddleware
 from .exceptions import ErrorView, ExceptionContextManager, HTTPException
@@ -108,9 +102,9 @@ class Index:
         on_startup: List[NoArgumentCallable] = [],
         on_shutdown: List[NoArgumentCallable] = [],
         routes: Iterable[BaseRoute] = [],
-        exception_handlers: Dict[int | Type[Exception], ErrorView] = {},
+        exception_handlers: Mapping[int | Type[BaseException], ErrorView] = {},
         factory_class: FactoryClass = FactoryClass(),
-        response_converters: Dict[type, Callable[..., HttpResponse]] = {},
+        response_converters: Mapping[type, Callable[..., HttpResponse]] = {},
     ) -> None:
         self.debug = debug
         self.state = State()
@@ -124,11 +118,16 @@ class Index:
         self._asgi_middlewares: List[F] = []
         self.app_with_middlewares = self.build_app_with_middlewares()
 
-        if self.debug:
-            self.add_middleware(DebugMiddleware)
-
     def build_app_with_middlewares(self) -> ASGIApp:
-        return reduce(lambda a, m: m(a), reversed(self._asgi_middlewares), self.app)
+        internal_middlewares: List[F] = []
+        if self.debug:
+            internal_middlewares.append(F(DebugMiddleware))
+
+        return reduce(
+            lambda a, m: m(a),
+            reversed([*self._asgi_middlewares, *internal_middlewares]),
+            self.app,
+        )
 
     def add_middleware(self, middleware_class: type, **options: Any) -> None:
         """
@@ -179,6 +178,7 @@ class Index:
                 else:
                     return await response(scope, receive, send)
         finally:
+            await request.close()
             request_var.reset(token)
 
     async def websocket(self, scope: Scope, receive: Receive, send: Send) -> None:
@@ -201,7 +201,7 @@ class Index:
 
 
 def create_response_converter(
-    converters: Dict[type, Callable[..., HttpResponse]]
+    converters: Mapping[type, Callable[..., HttpResponse]]
 ) -> functools._SingleDispatchCallable[HttpResponse]:
     """
     Create a converter for convert response.
@@ -220,52 +220,19 @@ def create_response_converter(
 
     response_converter.register(type(None), _none)
     response_converter.register(HttpResponse, lambda x: x)
-    response_converter.register(
-        dict,
-        lambda content, status=200, headers=None: JSONResponse(
-            content, status, headers
-        ),
-    )
-    response_converter.register(
-        list,
-        lambda content, status=200, headers=None: JSONResponse(
-            content, status, headers
-        ),
-    )
-    response_converter.register(
-        tuple,
-        lambda content, status=200, headers=None: JSONResponse(
-            content, status, headers
-        ),
-    )
-    response_converter.register(
-        bytes,
-        lambda content, status=200, headers=None: PlainTextResponse(
-            content, status, headers
-        ),
-    )
-    response_converter.register(
-        str,
-        lambda content, status=200, headers=None: PlainTextResponse(
-            content, status, headers
-        ),
-    )
-    response_converter.register(
-        AsyncGeneratorType,
-        lambda content, status=200, headers=None: SendEventResponse(
-            content, status, headers
-        ),
-    )
+    response_converter.register(dict, JSONResponse)
+    response_converter.register(list, JSONResponse)
+    response_converter.register(tuple, JSONResponse)
+    response_converter.register(bytes, PlainTextResponse)
+    response_converter.register(str, PlainTextResponse)
+    response_converter.register(AsyncGeneratorType, SendEventResponse)
     response_converter.register(
         PurePath,
         lambda filepath, download_name=None: FileResponse(
             str(filepath), download_name=download_name
         ),
     )
-    response_converter.register(
-        URL,
-        lambda url, status=307, headers=None: RedirectResponse(url, status, headers),
-    )
+    response_converter.register(URL, RedirectResponse)
 
     for type_, converter in converters.items():
         response_converter.register(type_, converter)
