@@ -11,7 +11,8 @@ from typing import List, Union
 import click
 
 from .__version__ import __version__
-from .utils import F, import_module
+from .routing.commands import display_urls
+from .utils import F, import_from_string, import_module
 
 
 def execute(command: Union[List[str], str]) -> int:
@@ -27,7 +28,7 @@ def execute(command: Union[List[str], str]) -> int:
         process.terminate()
         process.wait()
 
-    signal.signal(signal.SIGINT, sigint_handler)
+    signal.signal(signal.SIGTERM, sigint_handler)
 
     while process.poll() is None:
         time.sleep(1)
@@ -41,13 +42,123 @@ def index_cli():
 
 
 try:
-    import uvicorn
+    import hypercorn
 except ImportError:
     pass
 else:
 
+    @click.command(help="use hypercorn to run Index.py application")
+    @click.option(
+        "--bind",
+        default="127.0.0.1:4190",
+        show_default=True,
+        help="A string of the form: HOST:PORT, unix:PATH, fd://FD.",
+    )
+    @click.option(
+        "--log-level",
+        type=click.Choice(["critical", "error", "warning", "info", "debug"]),
+        default="info",
+        show_default=True,
+    )
+    @click.option(
+        "--worker-class",
+        "-k",
+        default="asyncio",
+        type=click.Choice(["asyncio", "uvloop", "trio"]),
+        show_choices=True,
+        show_default=True,
+    )
+    @click.option(
+        "--configuration",
+        "-c",
+        type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True),
+    )
+    @click.argument("application")
+    def hypercorn_cli(
+        worker_class: str,
+        configuration: str,
+        application: str,
+        bind: str,
+        log_level: str,
+    ):
+        sys.path.insert(0, os.getcwd())
+        asgi_app = import_from_string(application)
+        config = hypercorn.Config()
+        if configuration is not None:
+            if configuration.endswith(".py"):
+                config.from_pyfile(configuration)
+            elif configuration.endswith(".toml"):
+                config.from_toml(configuration)
+            else:
+                click.secho(
+                    "Please use configuration file path endswith `.py` or `.toml`.",
+                    fg="red",
+                )
+                raise SystemExit(1)
+        config.bind = [bind]
+        config.loglevel = log_level.upper()
+        config.worker_class = worker_class
+
+        create_signal_handle = lambda shutdown_event: lambda sig, frame: (
+            setattr(asgi_app, "should_exit", True),  # type: ignore
+            shutdown_event.set(),
+        )
+        if worker_class == "uvloop":
+            import uvloop
+
+            uvloop.install()
+        if worker_class in ("asyncio", "uvloop"):
+            import asyncio
+
+            from hypercorn.asyncio import serve
+
+            loop = asyncio.get_event_loop()
+
+            shutdown_event = asyncio.Event(loop=loop)
+            for sig in {signal.SIGINT, signal.SIGTERM}:
+                signal.signal(sig, create_signal_handle(shutdown_event))
+
+            loop.run_until_complete(
+                serve(asgi_app, config, shutdown_trigger=shutdown_event.wait)  # type: ignore
+            )
+        else:
+            import trio
+            from hypercorn.trio import serve  # type: ignore
+
+            shutdown_event = trio.Event()
+            for sig in {signal.SIGINT, signal.SIGTERM}:
+                signal.signal(sig, create_signal_handle(shutdown_event))
+
+            trio.run(serve(asgi_app, config, shutdown_trigger=shutdown_event.wait))  # type: ignore
+
+    index_cli.add_command(hypercorn_cli, name="hypercorn")
+
+try:
+    import uvicorn
+except ImportError:
+    pass
+else:
+    from .applications import Index
+
+    # See https://stackoverflow.com/questions/58133694/graceful-shutdown-of-uvicorn-starlette-app-with-websockets
+    origin_handle_exit = uvicorn.Server.handle_exit
+
+    def handle_exit(self: uvicorn.Server, sig, frame):
+        application = self.config.loaded_app
+        while not isinstance(application, Index):
+            application = application.app
+        application.should_exit = True
+        return origin_handle_exit(self, sig, frame)
+
+    uvicorn.Server.handle_exit = handle_exit
+
     @click.command(help="use uvicorn to run Index.py application")
-    @click.option("--bind", default="127.0.0.1:4190", show_default=True)
+    @click.option(
+        "--bind",
+        default="127.0.0.1:4190",
+        show_default=True,
+        help="A string of the form: HOST:PORT, unix:PATH, fd://FD.",
+    )
     @click.option("--autoreload/--no-autoreload", default=True, show_default=True)
     @click.option(
         "--log-level",
@@ -60,16 +171,7 @@ else:
         sys.path.insert(0, os.getcwd())
 
         if bind.startswith("unix:"):
-            unix_path = bind[5:]
-            bind_config = {
-                "uds": (
-                    "/" + unix_path.lstrip("/")
-                    if unix_path.startswith("/")
-                    else unix_path
-                )
-                | F(os.path.normpath)
-                | F(os.path.abspath)
-            }
+            bind_config = {"uds": bind[5:] | F(os.path.normpath) | F(os.path.abspath)}
             if autoreload:
                 click.secho(
                     "Reload option doesnt work with unix sockets "
@@ -77,8 +179,13 @@ else:
                     fg="yellow",
                 )
         elif bind.startswith("fd://"):
-            click.secho("Unsupport bind fd:// when using `index-cli uvicorn`", fg="red")
-            raise SystemExit(1)
+            bind_config = {"fd": int(bind[5:])}
+            if autoreload:
+                click.secho(
+                    "Reload option doesnt work with fd "
+                    "in uvicorn: https://github.com/encode/uvicorn/issues/368",
+                    fg="yellow",
+                )
         else:
             if ":" in bind:
                 host, port = bind.split(":")
@@ -125,7 +232,12 @@ else:
         pass
 
     @gunicorn_cli.command(help="Run gunicorn")
-    @click.option("--bind", default="127.0.0.1:4190", show_default=True)
+    @click.option(
+        "--bind",
+        default="127.0.0.1:4190",
+        show_default=True,
+        help="A string of the form: HOST:PORT, unix:PATH, fd://FD.",
+    )
     @click.option("--autoreload/--no-autoreload", default=False, show_default=True)
     @click.option(
         "--log-level",
@@ -134,6 +246,12 @@ else:
         show_default=True,
     )
     @click.option("--workers", "-w", default=cpu_count(), show_default=True)
+    @click.option(
+        "--worker-class",
+        "-k",
+        default="uvicorn.workers.UvicornWorker",
+        show_default=True,
+    )
     @click.option("--daemon", "-d", default=False, is_flag=True, show_default=True)
     @click.option(
         "--configuration",
@@ -143,6 +261,7 @@ else:
     @click.argument("application")
     def start(
         workers: int,
+        worker_class: str,
         daemon: bool,
         configuration: str,
         application: str,
@@ -151,7 +270,7 @@ else:
         log_level: str,
     ):
         command = (
-            f"{sys.executable} -m gunicorn -k uvicorn.workers.UvicornWorker"
+            f"{sys.executable} -m gunicorn -k {worker_class}"
             + f" --bind {bind}"
             + f" --chdir {os.getcwd()}"
             + f" --workers {workers}"
@@ -160,7 +279,7 @@ else:
         )
         args = command.split(" ")
         if daemon:
-            args.extend("-D --log-file run.log".split(" "))
+            args.extend("-D --log-file gunicorn.log".split(" "))
         if autoreload:
             args.append("--reload")
         if configuration:
@@ -204,4 +323,7 @@ else:
 
     index_cli.add_command(gunicorn_cli, "gunicorn")
 
+
+index_cli.add_command(display_urls, "display-urls")
+
 import_module("commands")
