@@ -7,9 +7,9 @@ from typing import (
     Any,
     Awaitable,
     Callable,
-    Coroutine,
     Dict,
     List,
+    Mapping,
     Optional,
     Type,
     TypeVar,
@@ -17,20 +17,30 @@ from typing import (
     overload,
 )
 
-from baize.asgi import HTTPException, PlainTextResponse
+from baize.asgi import PlainTextResponse
+from baize.exceptions import HTTPException
 from pydantic import ValidationError
 from pydantic.json import pydantic_encoder
+from typing_extensions import Literal
 
 from .requests import request
 from .responses import HttpResponse
 
 
 class RequestValidationError(Exception):
-    def __init__(self, validation_error: ValidationError) -> None:
+    def __init__(
+        self,
+        validation_error: ValidationError,
+        in_: Literal["path", "query", "header", "cookie", "body"],
+    ) -> None:
         self.validation_error = validation_error
+        self.in_ = in_
 
     def errors(self) -> List[Dict[str, Any]]:
-        return self.validation_error.errors()
+        errors = self.validation_error.errors()
+        for error in errors:
+            error["in"] = self.in_  # type: ignore
+        return errors  # type: ignore
 
     def json(self, *, indent: Union[None, int, str] = 2) -> str:
         return json.dumps(self.errors(), indent=indent, default=pydantic_encoder)
@@ -58,43 +68,50 @@ class RequestValidationError(Exception):
                         "description": "error message",
                         "type": "string",
                     },
+                    "ctx": {
+                        "title": "Context",
+                        "description": "error context",
+                        "type": "string",
+                    },
+                    "in": {
+                        "title": "In",
+                        "type": "string",
+                        "enum": ["path", "query", "header", "cookie", "body"],
+                    },
                 },
                 "required": ["loc", "type", "msg"],
             },
         }
 
 
-Error = TypeVar("Error", bound=Exception)
+Error = TypeVar("Error", bound=BaseException)
 ErrorView = Callable[[Error], Awaitable[HttpResponse]]
-View = Callable[[], Coroutine[None, None, HttpResponse]]
 
 
 class ExceptionContextManager:
     def __init__(
         self,
-        handlers: Dict[Union[int, Type[Exception]], ErrorView] = None,
+        handlers: Mapping[Union[int, Type[BaseException]], ErrorView] = {},
     ) -> None:
         self._status_handlers: Dict[int, ErrorView] = {}
-        self._exception_handlers: Dict[Type[Exception], ErrorView] = {
+        self._exception_handlers: Dict[Type[BaseException], ErrorView] = {
             HTTPException: self.http_exception,
-            RequestValidationError: self.request_validation_error,
+            RequestValidationError: self.validation_error,
         }
-        if handlers is not None:
-            for key, value in handlers.items():
-                self.add_exception_handler(key, value)
+        for key, value in handlers.items():
+            self.add_exception_handler(key, value)
 
     def add_exception_handler(
         self,
-        exc_class_or_status_code: Union[int, Type[Exception]],
+        exc_class_or_status_code: Union[int, Type[BaseException]],
         handler: Callable,
     ) -> None:
         if isinstance(exc_class_or_status_code, int):
             self._status_handlers[exc_class_or_status_code] = handler
         else:
-            assert issubclass(exc_class_or_status_code, Exception)
             self._exception_handlers[exc_class_or_status_code] = handler
 
-    def _lookup_exception_handler(self, exc: Exception) -> Optional[Callable]:
+    def _lookup_exception_handler(self, exc: BaseException) -> Optional[Callable]:
         for cls in type(exc).__mro__:
             if cls in self._exception_handlers:
                 return self._exception_handlers[cls]
@@ -105,13 +122,13 @@ class ExceptionContextManager:
 
     @overload
     async def __aexit__(self, exc_type: None, exc: None, tb: None) -> None:
-        pass
+        ...
 
     @overload
     async def __aexit__(
         self, exc_type: Type[BaseException], exc: BaseException, tb: TracebackType
     ) -> bool:
-        pass
+        ...
 
     async def __aexit__(self, exc_type, exc, tb):
         if exc_type is None:
@@ -132,19 +149,19 @@ class ExceptionContextManager:
     async def http_exception(exc: HTTPException) -> HttpResponse:
         if exc.status_code in {204, 304}:
             return HttpResponse(status_code=exc.status_code, headers=exc.headers)
-
-        return PlainTextResponse(
-            content=(
+        else:
+            return request.app.response_converter(
                 exc.content
-                if isinstance(exc.content, (bytes, str))
-                else HTTPStatus(exc.status_code).description
-            ),
-            status_code=exc.status_code,
-            headers=exc.headers,
-        )
+                if exc.content is not None
+                else HTTPStatus(exc.status_code).description,
+                exc.status_code,
+                exc.headers,
+            )
 
-    @staticmethod
-    async def request_validation_error(exc: RequestValidationError) -> HttpResponse:
-        return PlainTextResponse(
-            exc.json(), status_code=422, media_type="application/json"
-        )
+    async def validation_error(self, exc: RequestValidationError) -> HttpResponse:
+        if exc.in_ == "path":
+            return await self.http_exception(HTTPException(status_code=404))
+        else:
+            return PlainTextResponse(
+                exc.json(), status_code=422, media_type="application/json"
+            )
