@@ -25,7 +25,13 @@ from . import specification as spec
 from .extra_docs import merge_openapi_info
 from .schema import schema_parameter, schema_request_body, schema_response
 
-TagDetail = TypedDict("TagDetail", {"description": str, "paths": Sequence[str]})
+
+class TagDetailOptional(TypedDict, total=False):
+    paths: Sequence[str]
+
+
+class TagDetail(TagDetailOptional):
+    description: str
 
 
 class OpenAPI:
@@ -37,6 +43,7 @@ class OpenAPI:
         tags: Dict[str, TagDetail] = {},
         template_name: Literal["redoc", "swagger", "rapidoc"] = "swagger",
         template: str = "",
+        reload: bool = True,
     ) -> None:
         if template == "":
             template = (
@@ -44,24 +51,29 @@ class OpenAPI:
             ).read_text(encoding="utf8")
 
         self.html_template = template
+        self.reload = reload
 
-        self.openapi: spec.OpenAPI = {
-            "openapi": "3.0.3",
-            "info": info,
-            "paths": {},
-            "tags": [
+        self.openapi = spec.OpenAPI(
+            openapi="3.0.3",
+            info=info,
+            paths={},
+            tags=[
                 {"name": tag_name, "description": tag_info.get("description", "")}
                 for tag_name, tag_info in tags.items()
             ],
-            "components": {
-                "schemas": {"RequestValidationError": RequestValidationError.schema()},
+            components={
+                "schemas": {
+                    "RequestValidationError": typing.cast(
+                        spec.Schema, RequestValidationError.schema()
+                    )
+                },
             },
-        }
+        )
         if security_schemes:
-            self.openapi["components"]["securitySchemes"] = security_schemes
+            self.openapi.get("components", {})["securitySchemes"] = security_schemes
         self.path2tag: Dict[str, List[str]] = {}
         for tag_name, tag_info in tags.items():
-            for path in tag_info["paths"]:
+            for path in tag_info.get("paths", []):
                 self.path2tag.setdefault(path, []).append(tag_name)
         self.definitions: dict = {}
 
@@ -118,10 +130,10 @@ class OpenAPI:
         update_definitions = lambda path_item, x: _definitions.update(x) or path_item
 
         # This is mypy check error, if you use pyright/pylance, this is all fine.
-        if hasattr(func, "__summary__") and isinstance(func.__summary__, str):  # type: ignore
-            result["summary"] = func.__summary__  # type: ignore
-        if hasattr(func, "__description__") and isinstance(func.__description__, str):  # type: ignore
-            result["description"] = func.__description__  # type: ignore
+        if hasattr(func, "__docs_summary__") and isinstance(func.__docs_summary__, str):  # type: ignore
+            result["summary"] = func.__docs_summary__  # type: ignore
+        if hasattr(func, "__docs_description__") and isinstance(func.__docs_description__, str):  # type: ignore
+            result["description"] = func.__docs_description__  # type: ignore
 
         if isinstance(func.__doc__, str):
             clean_doc = inspect.cleandoc(func.__doc__)
@@ -179,7 +191,7 @@ class OpenAPI:
         if result and path in self.path2tag:
             result["tags"] = self.path2tag[path]
 
-        result["tags"] = getattr(func, "__tags__", []) + result.get("tags", [])
+        result["tags"] = getattr(func, "__docs_tags__", []) + result.get("tags", [])
 
         # merge user custom operation info
         return (
@@ -196,9 +208,24 @@ class OpenAPI:
         openapi = copy.deepcopy(self.openapi)
         openapi["servers"] = [
             {
-                "url": f"{request.url.scheme}://{request.url.netloc}",
+                "url": "/",
                 "description": "Current server",
-            }
+            },
+            spec.Server(
+                url="{scheme}://{address}/",
+                description="Custom API Server Host",
+                variables={
+                    "scheme": {
+                        "default": request.url.scheme,
+                        "enum": ["http", "https"],
+                        "description": "http or https",
+                    },
+                    "address": {
+                        "default": request.url.netloc,
+                        "description": "api server's host[:port]",
+                    },
+                },
+            ),
         ]
         openapi["paths"], definitions = copy.deepcopy(self._generate_paths(request.app))
         for path_item in openapi["paths"].values():
@@ -206,7 +233,7 @@ class OpenAPI:
                 operation = typing.cast(spec.Operation, operation)
                 if "responses" not in operation:
                     operation["responses"] = {}
-        openapi["components"].setdefault("schemas", {}).update(definitions)
+        openapi.get("components", {}).setdefault("schemas", {}).update(definitions)
         return openapi
 
     @property
@@ -220,7 +247,11 @@ class OpenAPI:
         async def json_docs():
             openapi = self.create_docs(request)
             return JSONResponse(
-                openapi, headers={"hash": md5(json.dumps(openapi).encode()).hexdigest()}
+                openapi,
+                headers={
+                    "hash": md5(json.dumps(openapi).encode()).hexdigest(),
+                    "reload": str(self.reload).lower(),
+                },
             )
 
         async def heartbeat():
@@ -231,7 +262,7 @@ class OpenAPI:
                     "data": json.dumps(openapi),
                 }
                 while not request.app.should_exit:
-                    await asyncio.sleep(1)
+                    await asyncio.sleep(60)
 
             return g()
 
