@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import asyncio
 import json
 import typing
 from contextvars import ContextVar
@@ -9,16 +10,18 @@ from baize.asgi import HTTPConnection as BaiZeHTTPConnection
 from baize.asgi import Request as BaiZeRequest
 from baize.asgi import WebSocket as BaiZeWebSocket
 from baize.asgi import WebSocketDisconnect, WebSocketState
+from baize.datastructures import ContentType
 from baize.exceptions import HTTPException
 from baize.utils import cached_property
+from typing_extensions import Annotated
 
 if typing.TYPE_CHECKING:
-    from indexpy.applications import Index
+    from .applications import Index
 
 from .utils import State, bind_contextvar
 
 
-class HTTPConnection(BaiZeHTTPConnection, typing.MutableMapping):
+class HTTPConnection(BaiZeHTTPConnection, typing.MutableMapping[str, typing.Any]):
     def __setitem__(self, name: str, value: typing.Any) -> None:
         self._scope[name] = value
 
@@ -36,11 +39,18 @@ class HTTPConnection(BaiZeHTTPConnection, typing.MutableMapping):
 
 
 class HttpRequest(BaiZeRequest, HTTPConnection):
-    async def data(self) -> typing.Any:
+    async def data(
+        self,
+    ) -> Annotated[
+        typing.Any,
+        ContentType("application/json"),
+        ContentType("application/x-www-form-urlencoded"),
+        ContentType("multipart/form-data"),
+    ]:
         content_type = self.content_type
         if content_type == "application/json":
             return await self.json
-        elif str(content_type) in (
+        elif content_type in (
             "multipart/form-data",
             "application/x-www-form-urlencoded",
         ):
@@ -55,6 +65,23 @@ request = bind_contextvar(request_var)
 
 
 class WebSocket(BaiZeWebSocket, HTTPConnection):
+    async def is_disconnected(self) -> bool:
+        """
+        The method used to determine whether the connection is interrupted.
+
+        NOTE: The call may discard the information sent by the client.
+        """
+        if not hasattr(self, "_is_disconnected"):
+            self._is_disconnected = False
+
+        if not self._is_disconnected:
+            try:
+                message = await asyncio.wait_for(self._receive(), timeout=0.0000001)
+                self._is_disconnected = message.get("type") == "websocket.disconnect"
+            except asyncio.TimeoutError:
+                pass
+        return self._is_disconnected
+
     async def receive_json(self, mode: str = "text") -> typing.Any:
         assert mode in ("text", "binary")
         assert self.application_state == WebSocketState.CONNECTED
